@@ -243,9 +243,6 @@ class Model(object):
         and store that information.
         """
         timeseries_constraint = ['r']
-        # These can be numeric, avoiding true/false constraints.
-        # No c_eff due to disagreement with non-timeseries constraint
-        # c_e_cap_gross_net_rule.
         allowed_timeseries_constraints = ['r_eff', 'r_scale', 'r2_eff', 's_loss',
                                           'e_prod', 'e_con', 'e_eff', 'p_eff',
                                           'e_cap_min_use', 'e_ramping',
@@ -357,23 +354,40 @@ class Model(object):
         Distance calculated using vincenty inverse formula (given in utils module).
         """
         # Check if metadata & links are loaded
-        if 'metadata' not in self.config_model or 'links' not in self.config_model:
-            return
-        elif self.config_model.links:
-            for link, v in self.config_model.links.items():
-                for trans, v2 in self.config_model.links[link].items():
-                    # for a given link and transmission type (e.g. 'hvac'),
-                    # check if distance is set.
-                    if 'distance' not in self.config_model.links[link][trans]:
-                        # Links are given as 'a, b',
-                        # so need to split them into individuals
-                        links = link.split(',')
-                        # Find distance using vincenty func & metadata of lat-long
-                        loc_coords = self.config_model.metadata.location_coordinates
-                        dist = utils.vincenty(getattr(loc_coords, links[0]),
-                                              getattr(loc_coords, links[1]))
-                        # update config_model
-                        self.config_model.links[link][trans]['distance'] = dist
+        if (not self.config_model.get('metadata', None)
+            or not self.config_model.get('links', None)):
+            return None
+
+        # We continue if both links and metadata are defined in config_model
+        link_tech = set() # fill with 'link.tech' strings
+        link_tech_distance = set() # fill with 'link.tech' strings if 'link.tech.distance' exists
+
+        for key in self.config_model.links.as_dict_flat().keys():
+            link_tech.update(['.'.join(key.split('.',2)[:2])]) # get only 'link.tech'
+            if 'distance' in key:
+                link_tech_distance(['.'.join(key.split('.',2)[:2])]) # get only 'link.tech'
+            # create set with 'link.tech' strings where 'link.tech.distance' does not exist
+            fill_distance = link_tech.difference(link_tech_distance)
+        for i in fill_distance:
+            # Links are given as 'a,b', so need to split them into individuals
+            locs = i.split('.')[0].split(',')
+            # Find distance using vincenty func & metadata of lat-long,
+            # ignoring curvature if metadata is given as x-y, not lat-long
+            loc_coords = self.config_model.metadata.location_coordinates
+            loc1 = getattr(loc_coords, locs[0]) # look for first location in a,b
+            loc2 = getattr(loc_coords, locs[1]) # look for second location in a,b
+            if all(['lat' in key or 'lon' in key for key in # geographic
+                   loc_coords.as_dict_flat().keys()]):
+                dist = utils.vincenty([loc1.lat, loc1.lon], [loc2.lat, loc2.lon])
+            elif all(['x' in key or 'y' in key for key in # cartesian
+                     loc_coords.as_dict_flat().keys()]):
+                dist = np.sqrt((loc1.x - loc2.x)**2 + (loc1.y - loc2.y)**2)
+            else:
+                raise KeyError('unidentified coordinate system. Expecting data '
+                       'in the format {lat: N, lon: M} or {x: N, y: M} for '
+                       'user coordinate values of N, M.')
+            # update config_model
+            self.config_model.links.set_key(i+'.distance', dist)
 
     def get_t(self, timestamp, offset=0):
         """
@@ -510,11 +524,14 @@ class Model(object):
         """
         if y in self._sets['y_conversion_plus']:
             if level:  # Either 2 or 3
-                return self.get_option(y + '_'.join('.carrier', direction, str(level)))
+                return self.get_option(y + '_'.join('.carrier',
+                                                    direction, str(level)))
+            primary_carrier, all_carriers = self.get_cp_carriers(y,
+                                                           direction=direction)
             if primary:
-                return self.get_cp_carriers(y, direction=direction)[0]
+                return primary_carrier
             if all_carriers:
-                return self.get_cp_carriers(y, direction=direction)[1]
+                return all_carriers
 
         carrier = self.get_option(
             y + '.carrier', default=y + '.carrier_' + direction
@@ -647,7 +664,6 @@ class Model(object):
                 raise e('primary_carrier must be set for conversion_plus technology '
                         '`{}` as carrier_out contains multiple carriers'.format(y))
             other_c.update(c.keys())
-            other_c.remove(primary_carrier)
         elif direction == 'out' and isinstance(c, str):
             other_c.update([c])
             primary_carrier = c
@@ -658,7 +674,11 @@ class Model(object):
         other_c.update(c_2.keys() if isinstance(c_2, dict) else [c_2])
         other_c.update(c_3.keys() if isinstance(c_3, dict) else [c_3])
         other_c.discard(False) # if there is no secondary or tertiary carrier, they return False
-        return [primary_carrier, tuple(other_c)]
+        if len(other_c) == 1: # only one value
+            other_c = str(list(other_c)[0])
+        else:
+            other_c = tuple(other_c)
+        return primary_carrier, other_c
 
     def scale_to_peak(self, df, peak, scale_time_res=True):
         """Returns the given dataframe scaled to the given peak value.
